@@ -528,6 +528,7 @@ static void command(const char *uid, const char *cmd, unsigned timeout)
 extern "C" int main(int argc, char **argv)
 {
 	static const char *user = NULL;
+	static char *plugins = NULL;
 	static char *cfgfile = NULL;
 	static bool daemon = false;
 	static bool warned = false;
@@ -544,6 +545,7 @@ extern "C" int main(int argc, char **argv)
 	char *cp, *tokens;
 	char *args[65];
 	char tbuf[512];
+	const char *argv0 = argv[0];
 
 	corefiles();
 
@@ -567,6 +569,10 @@ extern "C" int main(int argc, char **argv)
 	if(cp)
 		cfgfile = strdup(cp);
 
+	cp = getenv("PLUGINS");
+	if(cp)
+		plugins = strdup(cp);
+
 	if(!getuid())
 		daemon = true;
 
@@ -620,6 +626,20 @@ extern "C" int main(int argc, char **argv)
 			continue;
 		} 
 
+		if(!stricmp(*argv, "-l") || !stricmp(*argv, "-plugins")) {
+			plugins = *(++argv);
+			if(!plugins) {
+				fprintf(stderr, "*** sipw: plugins option missing\n");
+				exit(-1);
+			}
+			continue;
+		}
+
+		if(!strnicmp(*argv, "-plugins=", 9)) {
+			plugins = *argv + 9;
+			continue;
+		} 
+
 		if(!stricmp(*argv, "-c") || !stricmp(*argv, "-config")) {
 			cfgfile = *(++argv);
 			if(!cfgfile) {
@@ -818,6 +838,9 @@ extern "C" int main(int argc, char **argv)
 		Thread::policy(SCHED_RR);
 #endif
 
+	if(plugins)
+		server::plugins(argv0, plugins);
+
 	if(daemon)
 		background(user, cfgfile, priority);
 	else
